package com.kogenta.kettle

import android.util.Log
import com.kogenta.kettle.common.logging.LogLevel
import com.kogenta.kettle.sdk.Kettle
import com.kogenta.kettle.common.model.DemographicProfile
import com.kogenta.kettle.common.model.KettleMessage
import com.kogenta.kettle.sdk.KettleMessagingCenter
import com.kogenta.kettle.sdk.KettleMetrics
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.json.JSONObject

class PinchPlugin : MethodCallHandler {
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "com.fluxloop.pinch/sdk")
            channel.setMethodCallHandler(PinchPlugin())
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        try {
            when (call.method) {
                "start" -> {
                    Kettle.start()
                    result.success(true)
                }
                "stop" -> {
                    Kettle.stop()
                    result.success(true)
                }
                "setMessagingId" -> {
                    val messagingId: String? = call.argument("messagingId")
                    Kettle.messagingId = messagingId
                    result.success(true)
                }
                "addCustomEvent" -> {
                    val type: String? = call.argument("type")
                    val json: String? = call.argument("json")
                    if (type == null || json == null) {
                        result.error("Received invalid data", call.method, null);
                    } else {
                        Kettle.addCustomEvent(type, json)
                        result.success(true)
                    }
                }
                "setMetadata" -> {
                    val type: String? = call.argument("type")
                    val json: String? = call.argument("json")
                    if (type == null || json == null) {
                        result.error("Received invalid data", call.method, null);
                    } else {
                        Kettle.setMetadata(type, json)
                        result.success(true)
                    }
                }
                "sendDemographicProfile" -> {
                    val birthYear: Int? = call.argument("birthYear")
                    val gender: String? = call.argument("gender")
                    if (birthYear == null || gender == null) {
                        result.error("Received invalid data", call.method, null);
                    } else {
                        val finalGender = when (gender) {
                            "male" -> DemographicProfile.Gender.MALE
                            "female" -> DemographicProfile.Gender.FEMALE
                            "unknown" -> DemographicProfile.Gender.UNKNOWN
                            else -> DemographicProfile.Gender.UNKNOWN
                        }
                        Kettle.sendDemographicProfile(DemographicProfile(birthYear = birthYear, gender = finalGender))
                        result.success(true)
                    }
                }
                "getStarted" -> {
                    result.success(Kettle.started)
                }
                "metric" -> {
                    sendMetric(call, result)
                }
                "setLogLevel" -> setLogLevel(call, result)
                "getPrivacyTermsUrl" -> result.success(Kettle.privacyTermsUrl)
                "getPrivacyDashboard" -> getPrivacyDashboard(call, result)
                "grant" -> grant(call, result)
                "revoke" -> revoke(call, result)
                "deleteCollectedData" -> Kettle.deleteCollectedData { result.success(it) }
                "getGrantedConsents" -> getGrantedConsents(result)
                "getApiKey" -> result.success(Kettle.apiKey)
                "setAdid" -> result.success(true)
                "startBluetoothProvider" -> {
                    Kettle.start(arrayOf(Kettle.Provider.BLUETOOTH))
                    result.success(true)
                }
                "startMotionProvider" -> {
                    Kettle.start(arrayOf(Kettle.Provider.MOTION))
                    result.success(true)
                }
                "startLocationProvider" -> {
                    Kettle.start(arrayOf(Kettle.Provider.LOCATION))
                    result.success(true)
                }
                "stopBluetoothProvider" -> {
                    Kettle.stop(arrayOf(Kettle.Provider.BLUETOOTH))
                    result.success(true)
                }
                "stopMotionProvider" -> {
                    Kettle.stop(arrayOf(Kettle.Provider.MOTION))
                    result.success(true)
                }
                "stopLocationProvider" -> {
                    Kettle.stop(arrayOf(Kettle.Provider.LOCATION))
                    result.success(true)
                }
                "requestBluetoothPermission" -> result.success(true)
                "requestMotionPermission" -> result.success(true)
                "getEnabledProviders" -> getEnabledProviders(result)
                "getActivityEvents" -> {
                    val fromTime: Long? = call.argument("fromTime")
                    val toTime: Long? = call.argument("toTime")
                    if (fromTime == null || toTime == null) {
                        result.error("Received invalid data", call.method, null);
                    } else {
                        getActivityEvents(fromTime, toTime, result)
                        result.success(true)
                    }
                }
                "anonymizeLocation" -> {
                    val latitude: Double? = call.argument("latitude")
                    val longitude: Double? = call.argument("longitude")
                    if (latitude == null || longitude == null || (latitude == 0.0 && longitude == 0.0)) {
                        result.error("Received invalid arguments", call.method, null)
                    } else {
                        anonymizeLocation(latitude, longitude, result)
                    }
                }
                else -> result.notImplemented()
            }
        } catch (e: Exception) {
            result.error("Exception encountered", call.method, e)
        }
    }


    private fun getGrantedConsents(result: Result) {
        val consents = arrayListOf<Int>()
        for (consent in Kettle.grantedConsents) {
            when (consent) {
                Kettle.Consent.ANALYTICS -> consents.add(1)
                Kettle.Consent.SURVEYS -> consents.add(2)
                Kettle.Consent.CAMPAIGNS -> consents.add(3)
                Kettle.Consent.ADS -> consents.add(4)
            }
        }
        return result.success(consents.toList())
    }

    private fun getEnabledProviders(result: Result) {
        val providers = arrayListOf<String>()
        for (provider in Kettle.enabledProviders) {
            when (provider) {
                Kettle.Provider.LOCATION -> providers.add("location")
                Kettle.Provider.BLUETOOTH -> providers.add("bluetooth")
                Kettle.Provider.MOTION -> providers.add("motion")
            }
        }
        return result.success(providers.toList())
    }

    private fun getActivityEvents(fromTime: Long, toTime: Long, result: Result) {
        Kettle.getActivityEvents(fromTime, toTime) { activityEvents ->
            result.success(activityEvents)
        }
    }

    private fun anonymizeLocation(latitude: Double, longitude: Double, result: Result) {
        Kettle.anonymizeLocation(latitude, longitude) { locationId -> result.success(locationId)}
    }

    private fun setLogLevel(call: MethodCall, result: Result) {
        when (call.argument("logLevel") as Int?) {
            0 -> Kettle.logLevel = LogLevel.EMPTY
            1 -> Kettle.logLevel = LogLevel.TRACE
            2 -> Kettle.logLevel = LogLevel.VERBOSE
            3 -> Kettle.logLevel = LogLevel.WARNING
            4 -> Kettle.logLevel = LogLevel.INFO
            else -> {
                result.error("Invalid logLevel supplied", call.method, null)
                return
            }
        }
        return result.success(true)
    }

    private fun getPrivacyDashboard(call: MethodCall, result: Result) {
        when (call.argument<Int?>("consentId")) {
            1 -> result.success(Kettle.getPrivacyDashboard(Kettle.Consent.ANALYTICS))
            2 -> result.success(Kettle.getPrivacyDashboard(Kettle.Consent.SURVEYS))
            3 -> result.success(Kettle.getPrivacyDashboard(Kettle.Consent.CAMPAIGNS))
            4 -> result.success(Kettle.getPrivacyDashboard(Kettle.Consent.ADS))
            else -> result.success(Kettle.getPrivacyDashboard(Kettle.Consent.ANALYTICS))
        }
    }

    private fun grant(call: MethodCall, result: Result) {
        when (call.argument<Int?>("consentId")) {
            1 -> Kettle.grant(arrayOf(Kettle.Consent.ANALYTICS))
            2 -> Kettle.grant(arrayOf(Kettle.Consent.SURVEYS))
            3 -> Kettle.grant(arrayOf(Kettle.Consent.CAMPAIGNS))
            4 -> Kettle.grant(arrayOf(Kettle.Consent.ADS))
            else -> result.error("Invalid consent supplied", call.method, null)
        }
        result.success(true)
    }

    private fun revoke(call: MethodCall, result: Result) {
        when (call.argument<Int?>("consentId")) {
            1 -> Kettle.revoke(arrayOf(Kettle.Consent.ANALYTICS)) { result.success(it) }
            2 -> Kettle.revoke(arrayOf(Kettle.Consent.SURVEYS)) { result.success(it) }
            3 -> Kettle.revoke(arrayOf(Kettle.Consent.CAMPAIGNS)) { result.success(it) }
            4 -> Kettle.revoke(arrayOf(Kettle.Consent.ADS)) { result.success(it) }
            else -> result.error("Invalid consent supplied", call.method, null)
        }
    }

    private fun sendMetric(call: MethodCall, result: Result) {
        var failed = false
        val type: Int? = call.argument("type")
        val action: Int? = call.argument("action")

        if (type == null || action == null) {
            result.success(false)
            return
        }

        // 1 == Onboarding
        val version: String? = call.argument("version")
        val requestType: Int? = call.argument("requestType")
        val accepted: Boolean? = call.argument("accepted")

        if (type == 1) {
            when (action) {
                1 -> KettleMetrics.Onboarding.notified(version = version ?: "0")
                2 -> KettleMetrics.Onboarding.notificationDismissed()
                3 -> KettleMetrics.Onboarding.started(version = version ?: "0")
                4, 5 -> {
                    if (requestType == null || accepted == null) {
                        failed = true
                    } else {
                        val requestedType = getRequestType(requestType)
                        if (requestedType != null) {
                            if (type == 4) {
                                KettleMetrics.Onboarding.requestedPermission(requestedType, accepted)
                            } else if (type == 5) {
                                KettleMetrics.Onboarding.requestedRuntimePermission(requestedType, accepted)
                            }
                        } else {
                            failed = true
                        }
                    }
                }
                6 -> KettleMetrics.Onboarding.cancelled()
                7 -> KettleMetrics.Onboarding.completed()
                else -> failed = true
            }
        }
        result.success(!failed)
    }

    private fun getRequestType(requestType: Int): KettleMetrics.Onboarding.RequestType? {
        return when (requestType) {
            1 -> KettleMetrics.Onboarding.RequestType.LOCATION
            2 -> KettleMetrics.Onboarding.RequestType.WIFI
            3 -> KettleMetrics.Onboarding.RequestType.BLUETOOTH
            4 -> KettleMetrics.Onboarding.RequestType.MOTION
            6 -> KettleMetrics.Onboarding.RequestType.SURVEYS
            7 -> KettleMetrics.Onboarding.RequestType.ANALYTICS
            8 -> KettleMetrics.Onboarding.RequestType.CAMPAIGNS
            9 -> KettleMetrics.Onboarding.RequestType.ADS
            10 -> KettleMetrics.Onboarding.RequestType.BIRTH_YEAR
            11 -> KettleMetrics.Onboarding.RequestType.GENDER
            12 -> KettleMetrics.Onboarding.RequestType.KETTLE
            else -> null
        }
    }
}
